/***********************************************************************
 *  $RCSfile$ 
 *
 *  Copyright (C) 2005 Novell, Inc.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Author: Brady Anderson <banderso@novell.com>
 *
 ***********************************************************************/

using System;

namespace Simias
{
	/// <summary>
	/// Interface for an external identity sync provider
	/// </summary>
	public interface IIdentitySyncProvider
	{
		#region Properties
		/// <summary>
		/// Gets the name of the provider.
		/// </summary>
		string Name { get; }

		/// <summary>
		/// Gets the description of the provider.
		/// </summary>
		string Description { get; }
		#endregion

		#region Public Methods
		/// <summary>
		/// Call to abort an in process synchronization
		/// </summary>
		/// <returns>N/A</returns>
		void Abort();
		
		/// <summary>
		/// Call to inform a provider to start a synchronization cycle
		/// </summary>
		/// <returns>True - provider successfully started a sync cycle, False - provider could
		/// not start the sync cycle.</returns>
		bool Start( Simias.IdentitySync.State State );
		#endregion
	}
}	
